# Usage

`grdp [command] [resource]`

Use `grdp` to initialize grdp-cli. It will ask for authentification. After authentification it generates two
config files in home directory. One names projects.json and contains projects list and one is for kubernetes.

`[command]` can be type of: 

Configuration commands:

- `update`      updates current configs
- `projects`    prints active project
- `help`        prints help

Basic Commands:
- `get`           Display one or many resources
- `delete`        Delete resources by file names, stdin, resources and names, or by resources and label selector
- `logs`          Print the logs for a container in a pod
- `cp`            Copy files and directories to and from containers
- `exec`          Execute a command in a container

`[resource]` can be type of kuberneetes resource:

- `pod`
- `service`
- `deployment`
- `ingress`
- `namespace`

## Examples
- `$ grdp` - Configure grdp-cli
- `$ grdp update` - Update grdp-cli configuration
- `$ grdp help` - Get grdp-cli help
- `$ kubectl get pods` - Show pods of current project
- `$ kubeclt get service` - Show services of project
- `$ kubectl get ingress` - Show ingress controller
- `$ kubectl logs <pod name>` - Sho logs of selected pod
- `$ kubectl exec -it <pod name>` bash - Execute a command in a selected pod 







import os
import signal
import subprocess
import sys

from os.path import expanduser
from time import sleep

hostName = '127.0.0.1'
serverPort = 65510
GITLAB_URL = 'https://gitlab.onix.ua'
AUTH_ADDRESS = 'https://grdp-cli-auth.staging.onix.ua'
GRDPCLI_HOME_DIR = os.path.join(expanduser("~"), '.kube')
GRDPCLI_PROJECT_JSON = 'projects.json'
GRDPCLI_CONFIG_JSON = 'config.json'
KUBECTL_CONFIG = os.path.join(GRDPCLI_HOME_DIR, 'config')

if not os.path.exists(GRDPCLI_HOME_DIR):
    os.mkdir(GRDPCLI_HOME_DIR)

kubectl_clusters_template = """apiVersion: v1
clusters:
- cluster:
    certificate-authority-data: {k8s_authority_data}
    server: {k8s_api_address}
  name: grdp-cluster
contexts:
- context:
    cluster: grdp-cluster
    namespace: {namespace_name}
    user: {namespace_name}
  name: {namespace_name}
current-context: {namespace_name}
kind: Config
preferences: {{}}
users:
- name: {namespace_name}
  user:
    token: {access_token}
"""



import logging

class CustomFormatter(logging.Formatter):

    grey = "\x1b[38;20m"
    yellow = "\x1b[33;20m"
    red = "\x1b[31;20m"
    bold_red = "\x1b[31;1m"
    reset = "\x1b[0m"
    green = "\x1b[38;5;115m"
    format = "%(asctime)s: [%(name)s] - %(message)s"

    FORMATS = {
        logging.DEBUG: grey + format + reset,
        logging.INFO: green + format + reset,
        logging.WARNING: yellow + format + reset,
        logging.ERROR: red + format + reset,
        logging.CRITICAL: bold_red + format + reset
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)

logger = logging.getLogger('GRDPCLI')
logger.setLevel(logging.INFO)

ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)

ch.setFormatter(CustomFormatter())
logger.addHandler(ch)


import webbrowser
import requests
import logging
import json

from http.server import BaseHTTPRequestHandler, HTTPServer
from urllib.parse import urlparse, parse_qs
from grdpcli.variable import *
from grdpcli.logger import logger


def checkKubectl():
    """
        Checking kubectl command.
    """
    APP_NAME = 'kubectl'
    try:
        devnull = open(os.devnull)
        subprocess.Popen([APP_NAME], stdout=devnull, stderr=devnull).communicate()
    except:
        return False
    return True

def getProjectsJson():
    return json.load(open(os.path.join(GRDPCLI_HOME_DIR, GRDPCLI_PROJECT_JSON)))

def getGRCPcliConfig():
    return json.load(open(os.path.join(GRDPCLI_HOME_DIR, GRDPCLI_CONFIG_JSON)))

def getProjectsId():
    return json.load(open(os.path.join(GRDPCLI_HOME_DIR, GRDPCLI_CONFIG_JSON)))['active_project']

def getProjectName(id):
    id = None if id == 'None' else id
    if not id:
        return None

    projects = getProjectsJson()
    for key in projects[id].keys():
        name = list(projects[id].keys())[0]
        if key == 'status':
            name = list(projects[id].keys())[1]
        break
    return name

def getNamespaceName(id):
    projects = getProjectsJson()
    id = str(id)
    for key in projects[id].keys():
        if key == 'status':
            key = list(projects[id].keys())[1]
        return projects[id][key]['K8S_NAMESPACE']

def checkToken(access_token):
    r = requests.get("{}/api/v4/projects".format(GITLAB_URL), headers={"Authorization": "Bearer {}".format(access_token)}).status_code
    if int(r) == 200:
        return True
    return False

def getAllProjects(access_token):
    PROJECT_LIST = []
    page = 0
    while True:
        page = page + 1
        projects_dict = requests.get("{}/api/v4/projects?page={}".format(GITLAB_URL, page), headers={"Authorization": "Bearer {}".format(access_token)}).json()
        if not len(projects_dict) > 0:
            break
        PROJECT_LIST = PROJECT_LIST + projects_dict
    return PROJECT_LIST

def getProjectVariables(access_token, project_id, project_name):
    REQUIRED_VARIABLES = ['K8S_NAMESPACE','K8S_HOSTNAME','K8S_ACCESS_TOKEN']
    VARIABLES_DICT, variables_json = {}, {}
    page = 0
    while True:
        page = page + 1
        variables_data = requests.get("{}/api/v4/projects/{}/variables?page={}".format(GITLAB_URL, project_id, page), headers={"Authorization": "Bearer {}".format(access_token)}).json()
        if 'message' in variables_data:
            logger.warn(f"Skipping project {project_name} due reason: {variables_data['message']}")
            return { project_name: {}, "status": "False" }
        if not len(variables_data):
            break
        for item in variables_data:
            variables_json[item['key']] = item['value']

    if 'message' in variables_json:
        return {project_name: {}, "status": "False"}

    for req_var in REQUIRED_VARIABLES:
        for gl_var in variables_json:
            if gl_var == req_var:
                variable_template = {
                    gl_var: variables_json[gl_var]
                  }
                VARIABLES_DICT.update(variable_template)
    required_variables = {
        project_name : VARIABLES_DICT,
            "status": "True"
        }
    f_var = [i for i in required_variables[project_name]]
    for p in REQUIRED_VARIABLES:
        if p not in f_var:
            logger.warn(f"Skipping project: {project_name} due reason: Didn't found required variables {','.join(REQUIRED_VARIABLES)}.")
            return {project_name: {}, "status": "False"}
    return required_variables

def cliConfigCreate(access_token):
    logger.info("Getting and generation projects configuration file, please wait ...")
    PROJECTS_LIST = getAllProjects(access_token)
    config_template = {}
    id = 0
    for project in PROJECTS_LIST:
        project_data = getProjectVariables(access_token, project['id'], project['path'])
        if json.loads(project_data['status'].lower()) == False:
            continue
        id = id + 1
        template = {
            id: project_data
        }
        config_template.update(template)

        with open(os.path.join(GRDPCLI_HOME_DIR, GRDPCLI_PROJECT_JSON), 'w') as f:
            json.dump(config_template, f, indent=4, sort_keys=True)

def createGRCPcliConfig(data):
    WASTE_KEYS = ['status', 'access_token']
    for k in WASTE_KEYS:
        try:
            del data[k]
        except:
            continue
    with open(os.path.join(GRDPCLI_HOME_DIR, GRDPCLI_CONFIG_JSON), 'w') as f:
        json.dump(data, f, indent=4, sort_keys=True)

def setActiveProject(id):
    config_data = getGRCPcliConfig()
    config_data['active_project'] = id
    createGRCPcliConfig(config_data)

def generateKubectlConfig():
    projects = json.load(open(os.path.join(GRDPCLI_HOME_DIR, GRDPCLI_PROJECT_JSON)))
    config = json.load(open(os.path.join(GRDPCLI_HOME_DIR, GRDPCLI_CONFIG_JSON)))

    id = getProjectsId()
    k8s_authority_data = config['k8s_authority_data']
    k8s_api_address = config['k8s_api_address']
    for key in projects[id].keys():
        if key == 'status':
            continue
        namespace_name = projects[id][key]['K8S_NAMESPACE']
        access_token = projects[id][key]['K8S_ACCESS_TOKEN']
        break

    kubectl_template = kubectl_clusters_template.format(namespace_name=namespace_name, k8s_authority_data=k8s_authority_data, k8s_api_address=k8s_api_address, access_token=access_token)
    with open(KUBECTL_CONFIG, "w") as config:
        config.write(kubectl_template.lstrip())

class callbackServer(BaseHTTPRequestHandler):
    def _set_headers(self):
        self.send_response(200, "ok")
        self.send_header('Access-Control-Allow-Origin', '*')
        self.send_header('Access-Control-Allow-Methods', 'GET, POST, OPTIONS')
        self.send_header("Access-Control-Allow-Headers", "X-Requested-With")
        self.end_headers()

    def do_OPTIONS(self):
        self._set_headers()

    def do_POST(self):
        post_data = self.rfile.read(int(self.headers['Content-Length']))
        response_data = json.loads(post_data.decode('utf-8'))

        self._set_headers()
        self.wfile.write(bytes("", "utf-8"))

        if checkToken(response_data["access_token"]):
            cliConfigCreate(response_data["access_token"])
        createGRCPcliConfig(response_data)

        if os.path.exists(os.path.join(GRDPCLI_HOME_DIR,GRDPCLI_PROJECT_JSON)) and os.path.exists(os.path.join(GRDPCLI_HOME_DIR,GRDPCLI_CONFIG_JSON)):
            logger.info("Projects configuration file generated successfully")
            raise SystemExit
        else:
            raise ValueError('Projects configuration file not found or broken')

    def log_message(self, format, *args):
        return



import subprocess
import os
import sys
import pkg_resources

from rich.console import Console
from rich.markdown import Markdown

from grdpcli import *
from grdpcli.logger import logger

console = Console()

def command_help():
    help_filepath = os.path.realpath(os.path.join(os.getcwd(), os.path.dirname(__file__)))
    with open(f"{help_filepath}/help.py", "r") as help_file:
         console.print(Markdown(help_file.read()))
    sys.exit(0)

def command_version():
    version = pkg_resources.require("grdpcli")[0].version
    logger.info(f"grdp-cli version: {version}")

def command_update():
    """
        Updating configurations
    """
    logger.info("Updating GRDPCLI config")
    os.remove(os.path.join(GRDPCLI_HOME_DIR, GRDPCLI_CONFIG_JSON))
    os.remove(os.path.join(GRDPCLI_HOME_DIR, GRDPCLI_PROJECT_JSON))
    logger.info("Authorization is required, please ENTER to open up the browser to login or CTRL+C to cancel:")
    WEBSERVER = HTTPServer((hostName, serverPort), callbackServer)
    webbrowser.open(AUTH_ADDRESS, new=0, autoraise=True)
    logger.info("Waiting for authorization is compleate ...")
    try:
        WEBSERVER.serve_forever()
    except KeyboardInterrupt:
        pass

def command_projects():
    """
        Getting list of projects.
        Prints a table with list of projects and generates config file.
    """
    projects = getProjectsJson()
    IDS_LIST = projects.keys()
    active_project_id = getGRCPcliConfig()['active_project']
    result = subprocess.Popen(
        ["git", "rev-parse", "--show-toplevel"], 
        stdout=subprocess.PIPE,
        stdin=subprocess.PIPE,
        stderr=subprocess.PIPE)
    output, error = result.communicate()
    output = os.path.basename(output.decode().replace("\n", ""))
    if output:
        for key, val in projects.items():
            keys = [key.lower() for key in val.keys()]
            if output in keys:
                logger.info("Found active project id using git repository")
                setActiveProject(key)
                active_project_id = getGRCPcliConfig()['active_project']
    active_project_id = None if active_project_id == 'None' else active_project_id

    while True:
        if active_project_id:
            id = getGRCPcliConfig()['active_project']
            break

        id = input("Please provide id of your project, current active project is `{}: {}`: ".format(active_project_id, getProjectName(active_project_id)))

        if id not in IDS_LIST:
            logger.error("Project id is incorrect, please provide correct value")
        else:
            setActiveProject(id)
            break
    logger.info("Current active project is `{}`".format(getProjectName(id)))
    generateKubectlConfig()

def commandHandler(commands):
    """
        Handler for cli commands.
    """
    SUBCOMMANDS = ['update','projects', 'help', '--help', 'version', '--version']
    if len(commands) > 1:
        if commands[1] in SUBCOMMANDS:
            if commands[1] == 'update':
                command_update()
            if commands[1] == 'help' or commands[1] == '--help':
                command_help()
            if commands[1] == 'projects':
                command_projects()
            if commands[1] == 'version' or commands[1] == '--version':
                command_version()
        else:
            del commands[0]
            options = " ".join(commands)

            active_project_id = getGRCPcliConfig()['active_project']
            active_project_id = None if active_project_id == 'None' else active_project_id

            if not active_project_id:
                command_projects()

            context = getNamespaceName(getGRCPcliConfig()['active_project'])
            exit(os.WEXITSTATUS(os.system('KUBECONFIG={} kubectl --context {} {}'.format(KUBECTL_CONFIG, context, options))))
    else:
        command_projects()



#!/usr/bin/env python
import os

from distutils.core import setup

package_version = os.environ.get('CI_COMMIT_TAG', '1.0.10')
package_name = 'grdpcli'

data_files=[
        ('grdpcli/__init__.py'),
        ('grdpcli/commands.py'),
        ('grdpcli/variable.py'),
        ('grdpcli/help.py'),
    ]

def package_files(data_files, directory_list):

    paths_dict = {}

    for directory in directory_list:

        for (path, directories, filenames) in os.walk(directory):

            for filename in filenames:

                file_path = os.path.join(path, filename)
                install_path = os.path.join('share', package_name, path)

                print(install_path)

                if install_path in paths_dict.keys():
                    paths_dict[install_path].append(file_path)

                else:
                    paths_dict[install_path] = [file_path]

    for key in paths_dict.keys():
        data_files.append((key, paths_dict[key]))

    return data_files

setup(name='grdpcli-test',
      version=package_version,
      description='Gitlab Rapid Development Platform CLI client',
      author='Anton Marusenko',
      author_email='anton.marusenko@onix-systems.com',
      url='https://onix-systems.com',
      data_files=package_files(data_files, ['grdpcli/']),
      install_requires=[
          'requests',
          'rich'
      ],
      scripts=["grdp"],
      python_requires='>=3'
     )
